import fdubath.entrelacs.*;
import fdubath.entrelacs.tilling.*;
/** 
* Here is a simple sample demonstrating how to use 
* the fdubath.entrelacs.tilling package. 
* 
* One geometry of each implemented kind is created 
* and Ropes are loaded from .till files.
*
* 2 verteices are picked and moved along a closed path.
* When a key is pressed the code swap from one geometry 
* to the next one. 
*
* As an alternative from loading ropes from files, 
* the code creating a rope is also included into this sample.
* Note that it is not not used in this sample but can be swithched 
* on by changing the parameter of loadGeometries() to false in setup().
* 
*                                   F. Dubath 22/01/2010
*/


/**
* the current geometry
*/
TillingBackgroundGeometry geometry=null;

/**
* Geometries list
*/
TillingBackgroundGeometry [] geometries=null;

/**
* index of the current geometry into the geometries list (array)
*/
int geometryCmp =-1;

/**
* one of the vertex to be moved
*/
Vertex vMoved;

/**
* second vertex to be moved
*/
Vertex v2Moved;

/**
* original position of the first vertex 
* (stored in order to be restored when changing to the next geometry)
*/
double [] originalPosition;

/**
* original position of the second vertex 
* (stored in order to be restored when changing to the next geometry)
*/
double [] originalPosition2;

/**
* vector giving the direction (and maximal amplitude) of the vertex displacement.
*/
double [] direction=new double[]{ 0.04,0.04};

/**
* frame compter 
*/
int time=0;

void setup(){
  // define the window size
  size(1000,1000);
  // Create the geometry
  loadGeometries(true);  // change the parameter in order to use the in code created rope
  // Set the first geometry
  setNextGeometry();
}

/**
* create the geometries, set the drawers and load the ropes.
*/
void loadGeometries(boolean useFile){
  geometries= new TillingBackgroundGeometry[]{
    new TillingP3BackgroundGeometry(), 
    new TillingP3m1BackgroundGeometry(), 
    new TillingP31mBackgroundGeometry(),
    new TillingP4BackgroundGeometry(), 
    new TillingP4mBackgroundGeometry(),  
    new TillingP6BackgroundGeometry(), 
    new TillingP6mBackgroundGeometry() };

  for (int i=0; i<geometries.length; i++){
    geometry=geometries[i];
    // Set the drawer
    geometry.drawer=new ProcessingDrawer();
    //Load the ropes 
    if ((!useFile) && (geometry instanceof  TillingP3BackgroundGeometry)){
      createRopeList((TillingP3BackgroundGeometry)geometry);
    }
    else{
      loadRopeList(geometry);
    }
  }
}

/**
* Loop on the geometries of the list
*/
void setNextGeometry(){
  geometryCmp++;
  // if the counter exceed the array length: back to the first geometry.
  if (geometryCmp==geometries.length){
    geometryCmp=0;
  }
  // restore the original vertices position
  if ((originalPosition!=null) && (originalPosition2!=null)){
    geometry.moveVertex(vMoved,new double[]{
      originalPosition[0] , originalPosition[1]     }
    );
    geometry.moveVertex(v2Moved,new double[]{ 
      originalPosition2[0] , originalPosition2[1]        }
    );
  }
  // set the new geometry
  geometry=geometries[geometryCmp];
  // pick the vertices
  v2Moved= (Vertex) geometry.getReadOnlyVertexList().get(1) ;
  vMoved= (Vertex) geometry.getReadOnlyVertexList().get(0) ;
  // store original position of the vertices
  originalPosition= new double[]{
    vMoved.getPositionX(),vMoved.getPositionY()          };
  originalPosition2= new double[]{
    v2Moved.getPositionX(),v2Moved.getPositionY()          };
}


void draw(){
  //change the geometry
  if(keyPressed){
    setNextGeometry();
  }
  // Moove the vertices
  geometry.moveVertex(vMoved,new double[]{
    originalPosition[0]+ sin((TWO_PI/10.0)*time)*direction[0] ,
    originalPosition[1]+ sin((TWO_PI/10.0)*time)*direction[1]          }
  );
  geometry.moveVertex(v2Moved,new double[]{
    originalPosition2[0]+ sin((TWO_PI/10.0)*time)*direction[0] ,
    originalPosition2[1]+ sin((TWO_PI/10.0)*time)*direction[1]          }
  );
  time++;

  // Clean the image
  background(255);
  
  // draw the ropes
  geometry.draw(
  new int[]{ 0,1000 }    // Origin of the drawing (0,1000)
  ,new int[]{ 1000,1000} // clip size (1000,1000)
  ,80);                  // size of the geometry (80)
}

/**
 * Loading the ropes from a file using  processing "createReader()" to access it.
 */
void loadRopeList(TillingBackgroundGeometry geometry){
  CustomFileReader reader = new CustomFileReader();
  reader.geometry=geometry;
  BufferedReader br=null;
  if (geometry instanceof TillingP3BackgroundGeometry)
    br= createReader("simple_p3.till"); 
  else if (geometry instanceof TillingP3m1BackgroundGeometry)
    br= createReader("Sample_p3m1.till");
  else if (geometry instanceof TillingP31mBackgroundGeometry)
    br=createReader("Sample_p31m.till");
  else if (geometry instanceof TillingP4BackgroundGeometry)
    br=createReader("simple_p4.till");
  else if (geometry instanceof TillingP4mBackgroundGeometry)
    br=createReader("Sample_p4m.till");
  else if (geometry instanceof TillingP6BackgroundGeometry)
    br=createReader("simple_p6.till");
  else if (geometry instanceof TillingP6mBackgroundGeometry)
    br=createReader("Sample_p6m.till");
  if( (br==null )||( ! reader.loadFromReader(br))){
    println("Unable to load the file");
  }
}


/**
 * Creating a rope by defining the vertices.
 */

void createRopeList(TillingP3BackgroundGeometry geometry){
  // set the verbose parameter to true so the methodes output into the console 
  // when an operation fail.
  geometry.verbose=true;
  // list of vertex
  Vertex v1=geometry.addVertex(0,new double[]{ 
    0.18,0.1            }
  );
  Vertex v2=geometry.addVertex(0,new double[]{ 
    0.55,0.76           }
  );
  Vertex v3=geometry.addVertex(0,new double[]{ 
    1.3,0.77            }
  );
  Vertex v4=geometry.addVertex(0,new double[]{ 
    0.9,0.13            }
  );
  Vertex v5=geometry.addVertex(0,new double[]{ 
    0.74,0.43           }
  );
  /// rope from vertices
  Rope rope= geometry.addLoopRope(v1,v4);
  geometry.addVertexToRope(rope,v2,v1);
  geometry.addVertexToRope(rope,v3,v2);
  geometry.addVertexToRope(rope,v5,v3);

  rope.setRopeRGBColor(new int[]{204,0,51  });
  rope.setLineWidth(0.05);
  rope.setIntersectionGap(0.0051);

}






